import pygame
from pygame.locals import *
from pgu import tilevid, timer, gui, vid, engine

import grid
import glob, os

screenWidth, screenHeight = 800,600
tileWidth, tileHeight = 16,16
blueprintOrigin = 200,0
blueprintSize = 600,600
blueprintRect = Rect(blueprintOrigin,blueprintSize)
SCROLL_SPEED = 16
WHITE = (255,255,255)
BLACK = (0,0,0)
gaugeTextColor = BLACK

flow_none, flow_in, flow_out, flow_both = range(4)


class EditorComponent:
    """en el futuro subclassear de EditorSprite?"""
    cost = 100
    
    def __init__(self, level=None, *args, **kwargs):
        self.broken = False
        self.images = []
        self.initializeImages()
        self.sprite = vid.Sprite(self.images[0],(-1000,-1000))
        kwargs["position"] = (-100,-100)
        self.gridComponent = self.gridClass(*args, **kwargs)
        self.makeArrows(level)
        self.updateImage(level)
        
    def makeArrows(self, level):
        self.slotArrows = {}
        for slot, position in self.gridComponent.getSlots():
            direction = self.gridComponent.getSlotRelativePos(slot)
            arrowPosition = level.vid.tile_to_view(position)
            flow = self.getFlow(self.gridComponent.component, slot)
            self.slotArrows[slot] = vid.Sprite(level.arrows[direction[0]][flow], arrowPosition)
    
    def getBaseImage(self):
        return self.images[self.gridComponent.layoutn%len(self.images)]

    def setSpritePosition(self, spritePosition, level):
        self.sprite.rect.topleft = spritePosition

    def updateImage(self, level):
        self.updateComponentImage(level)
        self.updateArrowsImages(level)

    def updateComponentImage(self, level):
        self.sprite.image = self.getBaseImage()

    def getFlow(self, component, slot):
        if not self.gridComponent.component:
                flow = flow_none
        else:
            slt = self.gridComponent.component[slot]
            if slt.in_value:
                if slt.out_value:
                    flow = flow_both
                else:
                    flow = flow_in
            elif slt.out_value:
                flow = flow_out
            else:
                flow = flow_none
        #print "gotflow", flow
        return flow
        
    def updateArrowsImages(self, level):
        for slot, position in self.gridComponent.getSlots():
            direction = self.gridComponent.getSlotRelativePos(slot)
            arrowPosition = level.vid.tile_to_view(position)
            flow = self.getFlow(self.gridComponent.component, slot)
            self.slotArrows[slot].setimage(level.arrows[direction[0]][flow])
        
    def initializeImages(self):
        for imageFile in self.imageFiles:
            image = pygame.image.load("data/components/"+imageFile).convert_alpha()
            self.images.append(image)

    def showArrows(self, level):
        for slot, position in self.gridComponent.getSlots():
            arrowPosition = level.vid.tile_to_view(position)
            self.slotArrows[slot].rect.topleft = arrowPosition
        level.appendComponentArrows(self)

    def hideArrows(self, level):
        level.removeComponentArrows(self)

    def click(self, position, level):
        pass

    def loop(self, level):
        if level.state == level.STARTED:
            if not self.broken and self.gridComponent.isBroken():
                self.broken = True
                level.setBroken(self)
            self.showArrows(level)
        else:
            self.hideArrows(level)
            if self.broken:
                self.broken = False
                level.resetBroken(self)
                

class EditorSprite:
    def __init__(self, level=None, *args, **kwargs):
        self.images = []
        self.initializeImages()
        self.sprite = vid.Sprite(self.images[0],(-1000,-1000))
        kwargs["position"] = (-100,-100)
        self.updateImage(level)
        
    def getBaseImage(self):
        return self.images[self.gridComponent.layoutn%len(self.images)]

    def setSpritePosition(self, spritePosition, level):
        self.sprite.rect.topleft = spritePosition

    def updateImage(self, level):
        self.updateComponentImage(level)

    def updateComponentImage(self, level):
        self.sprite.image = self.getBaseImage()
        
    def initializeImages(self):
        for imageFile in self.imageFiles:
            image = pygame.image.load("data/components/"+imageFile).convert_alpha()
            self.images.append(image)

    def loop(self, level):
        pass
                
class VaporSprite(EditorSprite):
    imageFiles = map ( lambda n:"vapor%d.png"%n, range(6) ) 
    sounds = []
    gridClass = None

    def __init__(self, *args, **kwargs):
        self.vaporStill = 0
        EditorSprite.__init__(self, *args, **kwargs)
        
    def updateComponentImage(self, level):
        image = self.images[self.vaporStill%len(self.images)]
        self.sprite.image = image.subsurface((0,0,tileWidth,tileHeight))
        self.vaporStill+=1
        
    def loop(self, level):
        EditorSprite.loop(self, level)
        self.updateComponentImage(level)

class BoilerEditorComponent(EditorComponent):
    imageFiles = ["boiler.png"]
    sounds = []
    cost = 650
    gridClass = grid.BoilerGridComponent

class HorizontalPipeEditorComponent(EditorComponent):
    imageFiles = ["pipeline.png"]
    sounds = []
    gridClass = grid.HorizontalPipeGridComponent

    _costPerUnit = 10
    @property
    def cost(self):
        return self._costPerUnit+(self._costPerUnit * self.getLength())
    
    def initializeImages(self):
        EditorComponent.initializeImages(self)
        self.maxLength = self.images[0].get_width()

    def getLength(self):
        return self.gridComponent.size[0]

    def updateComponentImage(self, level):
        image = self.images[self.gridComponent.layoutn%len(self.images)]
        totalWidth = self.getLength()*tileWidth
        if totalWidth > self.maxLength:
            totalWidth = self.maxLength
        self.sprite.image = image.subsurface((0,0,totalWidth,tileHeight))

class VerticalPipeEditorComponent(EditorComponent):
    imageFiles = ["pipelinev.png"]
    sounds = []
    gridClass = grid.VerticalPipeGridComponent
    
    _costPerUnit = 10
    @property
    def cost(self):
        return self._costPerUnit+(self._costPerUnit * self.getLength())

    def initializeImages(self):
        EditorComponent.initializeImages(self)
        self.maxLength = self.images[0].get_height()

    def getLength(self):
        return self.gridComponent.size[1]

    def updateComponentImage(self, level):
        image = self.images[self.gridComponent.layoutn%len(self.images)]
        totalHeight = self.getLength()*tileHeight
        if totalHeight > self.maxLength:
            totalHeight = self.maxLength
        self.sprite.image = image.subsurface((0,0,tileWidth,totalHeight))
    
class GaugeEditorComponent(EditorComponent):
    imageFiles = ["gauge.png"]
    sounds = []
    gridClass = grid.GaugeGridComponent
    font = None
    cost = 60
    
    def __init__(self, *a, **kw):
        if self.font is None:
            GaugeEditorComponent.font = pygame.font.Font("data/airstrip.ttf", 12)
        EditorComponent.__init__(self, *a, **kw)

    def loop(self, level):
        EditorComponent.loop(self, level)
        self.updateComponentImage(level)

    def updateComponentImage(self, level):
        text = str(self.gridComponent.getValue())
        image = self.images[self.gridComponent.layoutn%len(self.images)].copy()
        textSurface = self.font.render( text, True, gaugeTextColor )
        srcRect=textSurface.get_rect()
        srcRect.center = image.get_rect().center
        image.blit(textSurface,srcRect)
        self.sprite.image = image

class TEditorComponent(EditorComponent):
    imageFiles = ["t.png", "t2.png", "t3.png", "t4.png"]
    sounds = []
    cost = 60
    gridClass = grid.TGridComponent

class XEditorComponent(EditorComponent):
    imageFiles = ["x.png"]
    sounds = []
    cost = 80
    gridClass = grid.XGridComponent
    
class ElbowEditorComponent(EditorComponent):
    imageFiles = ["Elbow.png", "Elbow2.png", "Elbow3.png", "Elbow4.png"]
    sounds = []
    cost = 40
    gridClass = grid.ElbowGridComponent
    
class ManualGearEditorComponent(GaugeEditorComponent):
    imageFiles = ["manualgear.png"]
    sounds = []
    cost = 100
    gridClass = grid.ManualGearGridComponent
    def click(self, position, level):
        dx = position[0] - self.sprite.rect.centerx + level.vid.view.left
        self.gridComponent.setValue(self.gridComponent.getValue() + dx/4)
    
class GearValveEditorComponent(EditorComponent):
    imageFiles = ["gearval.png"]
    sounds = []
    cost = 200
    gridClass = grid.GearValveGridComponent

class PressureToGearEditorComponent(EditorComponent):
    imageFiles = ["presuregear.png"]
    sounds = []
    cost = 300
    gridClass = grid.PressureToGearGridComponent
    
class TankEditorComponent(EditorComponent):
    imageFiles = ["tank.png"]
    sounds = []
    cost = 300
    gridClass = grid.TankGridComponent
    
class SmallTankEditorComponent(EditorComponent):
    imageFiles = ["smalltank.png"]
    sounds = []
    cost = 300
    gridClass = grid.SmallTankGridComponent
    
class MachineEditorComponent(EditorComponent):
    imageFiles = ["x.png"]
    sounds = []
    gridClass = grid.MachineGridComponent


#----------------------------------------------------------------------------------------------

class Help(engine.State):
    background = os.path.join("data", "help.jpg")
    background2 = os.path.join("data", "help2.jpg")
    sound_file = os.path.join("data", "menu.ogg")
    
    def init(self):
        self.next_state = None
        self.image = pygame.transform.scale(pygame.image.load(self.background), 
                              (800, 600))
        self.image2 = pygame.transform.scale(pygame.image.load(self.background2), 
                              (800, 600))
        self.sound = pygame.mixer.Sound( self.sound_file )
        self.sound.play(-1)
        self.returnTo.value["next_state"] = self.next
        
        self.returnTo.quit = 0
        self.state = 1
        
        
    def event(self, evt):
        if self.state == 2:
            if evt.type is KEYDOWN:
                self.sound.stop()
                self.returnTo.music.play()
                return self.returnTo
        elif self.state == 1:
            if evt.type is KEYDOWN:
                self.image = self.image2
                self.state = 2
    
        
    def loop(self):
        if self.next_state:
            self.sound.stop()
            return self.next_state
            
    def paint(self, screen):
        screen.blit(self.image,(0,0))
        pygame.display.flip()
        
    def update(self, screen):
        self.paint(screen)


 
    
class LevelEditor(engine.State):
    STARTED, STOP, RUNNING = range(3)
    
    def init(self):
        
        self.state = self.STOP
        self.researchCost = 0
        self.start_time = pygame.time.get_ticks()
        self.averageRunTime = None
        self.components = {}
        self.otherSprites = {}
        self.dx, self.dy = 0, 0
        self.quit = False
        self.vid = tilevid.Tilevid()
        
        self.soundDejar = pygame.mixer.Sound("data/sounds/deja_comp.wav")
        self.soundError = pygame.mixer.Sound("data/sounds/error.wav")
        self.soundNoanda = pygame.mixer.Sound("data/sounds/noanda.wav")
        self.music = pygame.mixer.Sound( "data/sounds/Game Music.ogg") 
        self.music.play()
                                        
        #idata = [
        #    ('toolbar', 'data/toolbar.png', (4,4,10,10)),
        #]
        #vid.load_images(idata)
        #toolbarSurface = pygame.image.load("data/toolbar.png").convert_alpha()
        #vid.screen.blit(toolbarSurface,(0,0))
        #toolbarSurface.blit(vid.screen,(0,0))
        self.panel = Panel(self)
        self.backgroundSurface = pygame.image.load("data/fondo.jpg").convert()
        self.panelSurface = pygame.image.load("data/label.png").convert()
        self.logoSurface = pygame.image.load("data/logo.png").convert_alpha()
       
        self.level = self.value["level"](self, self.vid)
        self.grid = self.level.getGrid()

        self.arrows = {}
        
        doarrow = lambda x: pygame.image.load( "data/components/arrow-%s.png"%x ).convert_alpha()
        tp = doarrow("top")
        bt = doarrow("bottom")
        lf = doarrow("left")
        rt = doarrow("right")
        ud = doarrow("top-bottom")
        lr = doarrow("left-right")
        no = doarrow("none")
        for direction in "top bottom left right".split():
            self.arrows[direction] = pygame.image.load( "data/components/arrow-%s.png"%direction ).convert_alpha()
        # orden flow_none, flow_in, flow_out, flow_both
        self.arrows = dict(
            top = [ no, bt, tp, ud],
            bottom =[ no, tp, bt, ud],
            left = [ no, rt, lf, lr],
            right = [ no, lf, rt, lr],
            )
        
    def playDejarComponente(self):
        self.soundDejar.play()

    def playNoanda(self):
        self.soundNoanda.play()

    def playError(self):
        self.soundError.play()

    def addResearchCost(self, cost):
        self.researchCost += cost
    def getResearchCost(self):
        return self.researchCost
    
    def getDevelopmentTime(self):
        return pygame.time.get_ticks()-self.start_time
    
    def getReplicationCost(self):
        return reduce(
            lambda x,y: x + y.cost, self.components.values(), 0
        )
        
    def getAverageRunTime(self):
        return self.averageRunTime
    
    def setAverageRunTime(self, runTime):
        self.averageRunTime = runTime
        if runTime is not None:
            self.quit = 1
    
    def doHelp(self):
        Help.next = self.value["next_state"]
        Help.returnTo = self
        self.value["next_state"] = Help
        self.quit = 1
        
        
    def toolFinished(self):
        self.panel.toolGroup.widgets[0].click()
        
    def appendComponent(self, component):
        self.averageRunTime = None
        self.components[component.gridComponent]=component
        self.vid.sprites.append(component.sprite)
        self.appendComponentArrows(component)

    def setBroken(self, component):
        vc = VaporSprite(level=self)
        self.otherSprites[component]=vc
        self.vid.sprites.append(vc.sprite)
        vPos = self.vid.tile_to_view(component.gridComponent.position)
        vc.setSpritePosition(vPos, self)
        
    def resetBroken(self, component):
        self.rmOtherSpritesFor(component)
                
    def appendComponentArrows(self, component):
        for arrow in component.slotArrows.values():
            if arrow not in self.vid.sprites:
                self.vid.sprites.append(arrow)

    def moveComponent(self, component, screenPosition):
        self.averageRunTime = None
        tilePosition = self.vid.screen_to_tile(screenPosition)
        spritePosition = self.vid.tile_to_view(tilePosition)
        if self.grid.tryComponent(component.gridComponent, tilePosition):
            cursor = pygame.cursors.arrow
            position_ok = True
        else:
            cursor = pygame.cursors.broken_x
            position_ok = False
        pygame.mouse.set_cursor(*cursor)
        component.setSpritePosition(spritePosition, self)
        return position_ok

    def tile_to_screen(self, tilePosition):
        return self.vid.tile_to_screen(tilePosition)

    def screen_to_tile(self, screenPosition):
        return self.vid.screen_to_tile(screenPosition)
        
    def getComponentAt(self, screenPosition, checkPhantasm = False):
        tilePosition = self.vid.screen_to_tile(screenPosition)
        gridComponent = self.grid.getComponentAt(tilePosition, checkPhantasm)
        return self.components.get(gridComponent, None)

    def placeDown(self, component, screenPosition):
        self.averageRunTime = None
        tilePosition = self.vid.screen_to_tile(screenPosition)
        if self.grid.tryComponent(component.gridComponent, tilePosition):
            self.grid.addComponent(component.gridComponent, tilePosition)
            return True
        else:
            return False

    def rotateComponent(self, component):
        self.averageRunTime = None
        self.grid.rotateComponent(component.gridComponent)
        component.updateImage(self)
        
    def removeComponent(self, component):
        self.averageRunTime = None
        del self.components[component.gridComponent]
        self.vid.sprites.remove(component.sprite)
        self.removeComponentArrows(component)
        self.rmOtherSpritesFor(component)
    
    def rmOtherSpritesFor(self, component):
        try:
            vc = self.otherSprites[component]
        except: 
            return
        self.vid.sprites.remove(vc.sprite)
        del self.otherSprites[component]
        
    def removeComponentArrows(self, component):
        for arrow in component.slotArrows.values():
            if arrow in self.vid.sprites:
                self.vid.sprites.remove(arrow)

    def killComponent(self, component):
        self.averageRunTime = None
        self.removeComponent(component)
        self.grid.removeComponent(component.gridComponent)

    def paintBackground(self, blueprintSurface):
        if self.vid.bounds != None: self.vid.view.clamp_ip(self.vid.bounds)
        vx, vy = self.vid.view.topleft
        bgX, bgY = self.backgroundSurface.get_size()
        for i in range(2):
            for j in range(2):
                dx, dy = (300-vx)%bgX-(j*bgX), (300-vy)%bgY-(i*bgY)
                blueprintSurface.blit(self.backgroundSurface,(dx,dy))

    def paint(self, screen):
        #self.paintToolbar(screen)
        screen.subsurface((0,0,200,600)).blit(self.panelSurface,(0,0))
        #screen.subsurface((200,0,600,600)).fill((96,96,96))
        #vid.screen.blit(toolbarSurface,(0,0))
        blueprintSurface = screen.subsurface((200,0,600,600))
        self.paintBackground(blueprintSurface)
        screen.subsurface((0,0,200,600)).blit(self.panelSurface,(0,0))
        self.panel.paint(screen.subsurface((0,0,200,600)))
        self.vid.paint(screen.subsurface((200,0,600,600)))
        blueprintSurface.blit(self.logoSurface,(blueprintSurface.get_width()-self.logoSurface.get_width(),0))
        pygame.display.flip()

    def update(self, screen):
        self.paint(screen)

    def loop(self):
        if (self.dx or self.dy) and blueprintRect.collidepoint(pygame.mouse.get_pos()):
            self.vid.view.x += self.dx
            self.vid.view.y += self.dy
            evt = pygame.event.Event(MOUSEMOTION, {"pos": pygame.mouse.get_pos(), "buttons": pygame.mouse.get_pressed(), "rel": pygame.mouse.get_rel()})
            pygame.event.post(evt)

        for component in self.components.values():
            component.loop(self)
        for sprite in self.otherSprites.values():
            sprite.loop(self)
            
        if self.state == self.RUNNING:
            self.grid.step(1)
            
        if self.state != self.STOP:
            for comp in self.components.values():
                comp.updateImage(self)
        
        self.vid.loop()
        if self.quit:        
            self.music.stop()
            if self.averageRunTime<>None:
                return self.value["score_state"](self.game, 
                    { "scores": self.getScore(), "level":self.level } )
            return self.value["next_state"](self.game)

    def getScore(self):
        scores = {'average':    self.getAverageRunTime(),
                  'research':   self.getResearchCost(),
                  'development':self.getDevelopmentTime(),
                  'replication':self.getReplicationCost() }
        overall = scores['development']*100
        overall+= scores['research']
        overall+= scores['replication']*10
        overall+= scores['average']
        scores['overall']=overall
        return scores

    def doQuit(self):
        self.quit = True

    def event(self, event):
        keyhandlers = {
            K_ESCAPE: (self.doQuit, () ),
        }
        if event.type is QUIT:
            self.doQuit()
        if event.type is KEYDOWN:
            function, params = keyhandlers.get(event.key, (None, None))
            if function:
                function(*params)
        self.panel.event(event, self)

class Tool:
    iconFile="selection.png"
    def __init__(self):
        self.icon = pygame.image.load( "data/botones/" + self.iconFile ).convert_alpha()
        pygame.mouse.set_cursor(*pygame.cursors.arrow)
        
    def clicked(self, level):
        pass
    def unclicked(self, level):
        pass
    def event(self, e, level):
        if e.type is MOUSEMOTION:
            pass


            
class SelectionTool(Tool):
    iconFile="selection.png"
    def event(self, e, level):
        if hasattr(e, "pos"):
            screenPosition = translate(e.pos, blueprintOrigin)
            component = level.getComponentAt(screenPosition)
            if component is not None:
                if e.type is MOUSEBUTTONDOWN and e.button==1:
                    component.click(screenPosition, level)
                cursor = pygame.cursors.arrow
            else:
                cursor = pygame.cursors.broken_x
            pygame.mouse.set_cursor(*cursor)

class RotationTool(Tool):
    iconFile="rotation.png"
    def event(self, e, level):
        if hasattr(e, "pos"):
            screenPosition = translate(e.pos, blueprintOrigin)
            component = level.getComponentAt(screenPosition)
            if component is not None:
                if e.type is MOUSEBUTTONDOWN and e.button==1:
                    level.rotateComponent(component)
                cursor = pygame.cursors.arrow
            else:
                cursor = pygame.cursors.broken_x
            pygame.mouse.set_cursor(*cursor)

class CreationTool(Tool):
    def __init__(self, editorComponent):
        Tool.__init__(self)
        image = pygame.image.load("data/components/double/" + editorComponent.imageFiles[0]).convert_alpha()
        self.icon = pygame.transform.scale(image, image.get_rect().fit(0,0,64,48).bottomright)
        self.editorComponent = editorComponent

    def clicked(self, level):
        self.newComponent = self.editorComponent(level=level)
        level.appendComponent(self.newComponent)

    def unclicked(self, level):
        if self.newComponent:
            level.removeComponent(self.newComponent)
            self.newComponent = None

    def event(self, e, level):
        if not self.newComponent:
            return
        if hasattr(e, "pos"):
            screenPosition = translate(e.pos, blueprintOrigin)
            if e.type is MOUSEMOTION:
                level.moveComponent(self.newComponent, screenPosition)
            if e.type is MOUSEBUTTONDOWN and e.button==1:
                if level.placeDown(self.newComponent, screenPosition):
                    level.addResearchCost( self.newComponent.cost )
                    self.newComponent = None
                    level.toolFinished()
                    #self.group.value = 0
                    level.playDejarComponente()
                else:
                    level.playError()
           

class StrechCreationTool(Tool):
    START, END = range(2)
    
    def __init__(self, verticalEditorComponent, horizontalEditorComponent):
        Tool.__init__(self)
        self.icon = pygame.transform.scale(pygame.image.load("data/components/" + horizontalEditorComponent.imageFiles[0]).convert_alpha(), (64,64))
        self.verticalEditorComponent = verticalEditorComponent
        self.editorComponent = self.horizontalEditorComponent = horizontalEditorComponent
        self.state = self.START
        self.size = 1

    def clicked(self, level):
        self.editorComponent = self.horizontalEditorComponent
        self.makeComponent(level)
        
    def makeComponent(self, level):
        self.newComponent = self.editorComponent(level, self.size)
        level.appendComponent(self.newComponent)

    def unclicked(self, level):
        if self.newComponent:
            self.removeComponent(level)
        self.state = self.START
        self.size = 1
            
    def removeComponent(self, level):
        level.removeComponent(self.newComponent)
        self.newComponent = None
        
    def resizeComponent(self, level):
        if self.newComponent:
            self.removeComponent(level)
            xsize = abs(self.start_pos[0]-self.end_pos[0])+1
            ysize = abs(self.start_pos[1]-self.end_pos[1])+1
            if  ysize > xsize:
                self.editorComponent = self.verticalEditorComponent
                self.size = ysize
                if self.start_pos[1]>self.end_pos[1]:
                    self.top_left = (self.start_pos[0], self.end_pos[1])
                else:
                    self.top_left = self.start_pos
            else:
                self.editorComponent = self.horizontalEditorComponent
                self.size = xsize
                if self.start_pos[0]>self.end_pos[0]:
                    self.top_left = (self.end_pos[0],self.start_pos[1])
                else:
                    self.top_left = self.start_pos
                
            self.makeComponent(level)

    
    def event(self, e, level):
        if not self.newComponent:
            return
        if e.type is MOUSEMOTION:
            screenPosition = translate(e.pos, blueprintOrigin)
            if self.state == self.START:
                level.moveComponent(self.newComponent, screenPosition)
                self.start_pos = level.screen_to_tile(screenPosition)
            elif self.state == self.END:
                self.end_pos = level.screen_to_tile(screenPosition)
                self.resizeComponent(level)
                screenOriginPosition = level.tile_to_screen(self.top_left)
                level.moveComponent(self.newComponent, screenOriginPosition)
                
        if e.type is MOUSEBUTTONDOWN and e.button==1:
            screenPosition = translate(e.pos, blueprintOrigin)
            if self.state == self.END:
                position = level.tile_to_screen(self.top_left)
                if level.placeDown(self.newComponent, position):
                    level.addResearchCost( self.newComponent.cost )
                    self.newComponent = None
                    level.toolFinished()
                    level.playDejarComponente()
                else:
                    level.playError()

            elif self.state == self.START:
                self.start_pos = level.screen_to_tile(screenPosition)
                #print "START", self.start_pos
                if level.moveComponent(self.newComponent, screenPosition):
                    self.state = self.END
                    level.playDejarComponente()
                else:
                    level.playError()

class DeletionTool(Tool):
    iconFile="deletion.png"
    def event(self, e, level):
        if hasattr(e, "pos"):
            screenPosition = translate(e.pos, blueprintOrigin)
            component = level.getComponentAt(screenPosition)
            if component is None:
                component = level.getComponentAt(screenPosition, True)
            if component is not None:
                if e.type is MOUSEBUTTONDOWN and e.button==1:
                    level.killComponent(component)
                cursor = pygame.cursors.arrow
            else:
                cursor = pygame.cursors.broken_x
            pygame.mouse.set_cursor(*cursor)
 
class StartStopTool(Tool):
    def __init__(self, *a, **kw):
        self.icons = [ pygame.image.load("data/botones/%s.png"%filename).convert_alpha() for filename in ["stop", "start"] ] 
        self.icon = self.icons[1]

    def updateIcon(self, state):
        self.button.setwidget(gui.Image(self.icons[state]))
        self.button.resize()

    def clicked(self, level):
        if level.state == level.STOP:
            level.grid.start()
            level.grid.machine.dump()
            level.state = level.STARTED
        else:
            level.state = level.STOP
        self.updateIcon(level.state)
        level.toolFinished()
        
    
class RunPauseTool(Tool):
    iconFile="run.png"
    def clicked(self, level):
        if level.state == level.STOP:
            level.grid.start()
            level.grid.machine.dump()
            level.state = level.RUNNING
        else:
            if level.state == level.STARTED:
                level.state = level.RUNNING
            else:
                level.state = level.STARTED
        level.toolFinished()
        
        
class StepTool(Tool):
    iconFile="step.png"
    def clicked(self, level):
        if level.state == level.STOP:
            level.grid.start()
            level.grid.machine.dump()
            level.state = level.STARTED
        else:
            if level.state == level.RUNNING:
                level.state = level.STARTED
        level.grid.step(1)
        level.toolFinished()
    
class HelpTool(Tool):
    iconFile="help.png"
    def clicked(self, level):
        level.doHelp()
        level.toolFinished()
    
class ValidatorTool(Tool):
    iconFile="test.png"
    def clicked(self, level):
        #level.grid.dump(True)
         
        #pygame.mouse.set_cursor(*pygame.cursors.broken_x)
        level.grid.start()
        #level.grid.machine.dump()
        #level.grid.step(10)
        validationResult = level.level.runTestSuite()
        print "validation:", validationResult
        pygame.mouse.set_cursor(*pygame.cursors.arrow)
        pygame.mouse.set_cursor(*pygame.cursors.arrow)
        pygame.mouse.set_cursor(*pygame.cursors.arrow)
        
        if validationResult is None:
            level.playNoanda()
        level.setAverageRunTime(validationResult)
        
        level.toolFinished()

def translate(pos1, pos2):
    return pos1[0]-pos2[0], pos1[1]-pos2[1]

def getTools():
    return [
    SelectionTool(),
    RotationTool(),
    DeletionTool(),
    CreationTool(BoilerEditorComponent),
    CreationTool(GaugeEditorComponent),
    CreationTool(TEditorComponent),
    CreationTool(XEditorComponent),
    CreationTool(ElbowEditorComponent),
    CreationTool(ManualGearEditorComponent),
    CreationTool(GearValveEditorComponent),
    CreationTool(PressureToGearEditorComponent),
    CreationTool(TankEditorComponent),
    CreationTool(SmallTankEditorComponent),
    StrechCreationTool(VerticalPipeEditorComponent,HorizontalPipeEditorComponent),
    ValidatorTool(), HelpTool(),
    StartStopTool(), StepTool(), RunPauseTool(),
    
]

class Panel(gui.Desktop):
    selectedTool = None
    def __init__(self, level):
        self.level = level
        gui.Desktop.__init__(self, theme=gui.Theme(["data/themes/default"]))
        self.toolGroup = gui.Group(name='colors')
        table = gui.Table(width=200, height=600)
        for n, tool in enumerate( getTools() ):
            if( n%2 == 0 ):
                table.tr()
            button = gui.Tool(self.toolGroup, gui.Image(tool.icon), tool)
            tool.button = button
            table.td(button)

        self.toolbar = gui.Container(align=-1,valign=-1)
        self.toolbar.add(table,0,0)
        self.toolGroup.connect(gui.CHANGE, self.toolClicked, ())
        self.init(self.toolbar)

    def toolClicked(self, *a):
        if self.selectedTool is not None:
            self.selectedTool.unclicked(self.level)
        self.selectedTool = self.toolGroup.value
        self.selectedTool.clicked(self.level)

    def event(self, event, level):
        gui.Desktop.event(self, event)

        if self.selectedTool is not None:
            self.selectedTool.event(event, level)

        try:
            self.level.dx, self.level.dy = 0, 0
            x, y = event.pos 
            if x > 700: self.level.dx += SCROLL_SPEED
            if x > 100 and x < 300: self.level.dx -= SCROLL_SPEED
            if y > 500: self.level.dy += SCROLL_SPEED
            if y < 100: self.level.dy -= SCROLL_SPEED
        except:
            pass


if __name__ == "__main__":
    import levels
    class StimGame(engine.Game):
        def init(self):
            self.screen = pygame.display.set_mode( (800,600), SWSURFACE ) # | FULLSCREEN)
            pygame.font.init()
            pygame.mixer.init()
            self.timer = timer.Timer(20)
        def tick(self):
            self.timer.tick()
    game = StimGame()
    game.run(LevelEditor(game, dict(level=levels.LevelSix, next_state=engine.Quit)))
