import grid
from machine import PipeSlot, GearSlot
import machine
import pygame
import random
from pgu import vid

tileWidth, tileHeight = 16,16


class Level:
    max_test_steps = 10000
    
    def __init__(self, editor, myvid):
        self.vid = myvid
        myvid.tga_load_tiles('data/tiles.tga', (tileWidth, tileHeight))
        myvid.resize((self.levelWidth, self.levelHeight), True)
        myvid.bounds = pygame.Rect(0,0,self.levelWidth*tileWidth, self.levelHeight*tileHeight)
        
        
        for y in range(self.levelHeight):
            myvid.tlayer[y][0] = 2
            myvid.tlayer[y][self.levelWidth-1] = 2
        for x in range(self.levelWidth):
            myvid.tlayer[0][x] = 2
            myvid.tlayer[self.levelHeight-1][x] = 2
        
        levelSize = (self.levelWidth, self.levelHeight)
        ports = dict( [ (label, self.makePos(side, pos)) for label, (side, pos, kind, img) in self.ports.items() ] )
        pipe = pygame.image.load("data/components/pipe.png")
        labelTiles = []
        
        for label, (side, pos, kind, img) in self.ports.items():
            image = pygame.image.load(img)
            screenPos, tiles = self.makeLabelPos(side, pos, image)
            labelTiles += tiles
            pipePos = self.makePipePos(side, pos)            
            sprite = vid.Sprite(image, screenPos)
            self.vid.sprites.append( sprite ) 
            self.vid.sprites.append( vid.Sprite(pipe, pipePos ))

        self.grid = grid.Grid(levelSize, self.machineBuilder, ports, labelTiles)
        
    
        
    def makePos(self, side, pos):
        if side == "left": 
            return (1,pos)
        elif side == "top":
            return (pos, 1)
        elif side == "right":
            return (self.levelWidth-2,pos)
        elif side == "bottom":
            return (self.levelHeight-2, pos)
        raise Exception("No such side:"+side)
        
    def makePipePos(self, side, pos):
        if side == "left": 
            fpos = (0,pos)
        elif side == "top":
            fpos = (pos, 0)
        elif side == "right":
            fpos = (self.levelWidth-1,pos)
        elif side == "bottom":
            fpos = (self.levelHeight-1, pos)
        else:
            raise Exception("No such side:"+side)
        return self.vid.tile_to_screen( fpos )
    
    
    def makeLabelPos(self, side, pos, image):
        x_delta = image.get_width()/tileWidth
        y_delta = image.get_height()/tileHeight
        
        if side == "left": 
            fpos = (0,pos+1)
        elif side == "top":
            fpos = (pos+1, 0)
        elif side == "right":
            fpos = (self.levelWidth-x_delta,pos+1)
        elif side == "bottom":
            fpos = (self.levelHeight-y_delta, pos+1)
        else:
            raise Exception("No such side:"+side)
        
        tiles = [ ]
        px,py = fpos
        for x in range(image.get_width()/tileWidth):
            for y in range(image.get_height()/tileHeight):
                tiles += [ (px+x, py+y ) ]
        return self.vid.tile_to_screen(fpos), tiles

    def getGrid(self):
        return self.grid
    
    def machineBuilder(self):
        m = machine.Machine(
            dict([ (label, kind) for label, (side, pos, kind, img) in self.ports.items() ], ),
            0 # debug?
            )
        self.setInputs(m)
        m.setValidator( self.machineValidator )
        return m
    
    def setInputs(self, machine):
        for slot in self.ports:
            if hasattr(self, "setPort"+slot):
                machine[slot].out_value= getattr(self, "setPort"+slot)()

    def runTestSuite(self):
        total_time = 0
        machine = self.grid.machine
        valid_cases = 0
        
        for test in self.test_cases:
            machine.reset()
            for k,v in test.items():
                machine[k].out_value = v
                
            for step in xrange(self.max_test_steps):
                machine.step()
                
                if machine["done"].in_value == 1000:
                    if self.machineValidator( machine ):
                        total_time += step
                        valid_cases += 1
                        break
                    else:
                        return None
        if valid_cases != len(self.test_cases):
            print "valdiation error:", valid_cases, "of", len(self.test_cases)
            return None
        return total_time / len(self.test_cases)
                    

    def validate(self):
        return self.machineValidator(self.grid.machine)
                    
    def machineValidator(self, machine):
        pass
        
        
 
class LevelOne(Level):
    levelno = 1
    name = "Division by two"
    description = """
Build a machine that produces at "halfA"
half the value of "A"

In "Done" you must connect a 1000 psi pressure 
when the correct value is produced at "halfA".

Use gauges if you want to know how much 
pressure flows thru a pipe.

Use "step" to see the direction of flow as
its changes at every step.

To generate steam use a Boiler, it produces 1000psi
at its output.

T Components can split pressure or join pressures
depending on how many of their connectors
have incoming steam.

Steam flows one step at a time, so be sure to
check the debugger so timing is correct.

Conect all your components using pipes.

Press the "test" button when done, if the 
machine works, you will have finished 
the level. The test button is the one with 
the question mark.


now, press any key to start playing.

















PRESS A KEY!

"""
    levelWidth, levelHeight = 30,30
    
    test_cases = (
        dict(A=1),
        dict(A=10),
        dict(A=111),
        dict(A=100),
        dict(A=200),
        dict(A=333),
        dict(A=488),           
        )
                                
    ports = { 'A': ("left",5, PipeSlot, "data/labels/A.png"),
              'halfA': ("right",5, PipeSlot, "data/labels/halfA.png"),
              "done": ("right", 17,PipeSlot, "data/labels/done.png"),
           }
    def machineValidator(self, machine):
        print machine["A"].out_value, machine["A"].out_value/2, machine["halfA"].in_value
        return (machine["A"].out_value/2 == machine["halfA"].in_value)
    
    def setPortA(self):
        return random.randint(0, 499)
    
    
            
class LevelTwo(Level):
    levelno = 2
    name = "Addition"
    description = """
Build a machine that produces at "AplusB" the value
of "A" plus the value of "B".

Use the debugger to follow the steam as it moves 
through the pipes, and try not to break any 
components. If to much pressure enters a 
component  and theres no way for it to leave 
the component it will break.















you MUST press a key to continue
"""

    levelWidth, levelHeight = 30,30
    
    test_cases = (
        dict(A=1, B=1),
        dict(A=10, B=1),
        dict(A=1, B=10),
        dict(A=100, B=10),
        dict(A=200, B=1),
        dict(A=333, B=600),
        dict(A=234, B=432),           
        )
                                
    ports = { 'A': ("left",5, PipeSlot, "data/labels/A.png"),
              'B': ("left",11, PipeSlot, "data/labels/B.png"),
              "AplusB": ("right",5, PipeSlot, "data/labels/AplusB.png"),
              "done": ("right", 11,PipeSlot, "data/labels/done.png"),
           }
    def machineValidator(self, machine):
        print machine["A"].out_value,machine["B"].out_value,machine["A"].out_value+machine["B"].out_value,machine["AplusB"].in_value
        return (machine["A"].out_value+machine["B"].out_value ==machine["AplusB"].in_value)
    
    def setPortA(self):
        return random.randint(1, 500)
    setPortB = setPortA
        
class LevelFour(Level):
    levelno = 4
    name = "Copy"
    description = """
Build a machine that produces the value of "A"
both at "A1" and "A2".

Try to not to waste componentents in the 
construction of the machine, so you can score 
better with a lower research cost. Try using 
only the ones you think will make it to the 
final machine.
























Yes, we dont know physics, its just for phun
"""

    levelWidth, levelHeight = 30,30
    
    test_cases = (
        dict(A=1),
        dict(A=10),
        dict(A=111),
        dict(A=100),
        dict(A=200),
        dict(A=333),
        dict(A=888),           
        )
                                
    ports = { 'A': ("left",5, PipeSlot, "data/labels/A.png"),
              'A1': ("right",5, PipeSlot, "data/labels/A1.png"),
              "A2": ("right",11, PipeSlot, "data/labels/A2.png"),
              "done": ("right", 17,PipeSlot, "data/labels/done.png"),
           }
    def machineValidator(self, machine):
        return (machine["A"].out_value == machine["A1"].in_value and machine["A"].out_value == machine["A2"].in_value)
    
    def setPortA(self):
        return random.randint(0, 999)
    
class LevelThree(Level):
    levelno = 3
    name = "Substraction"
    description = """
Build a machine that produces at "AminusB"
the value of "A" minus the value of "B", 
or 0 if "B" is greater than "A".

Remember how you build this parts, 
as you will need them in future machines.

This is the first level in which you
will really need gears. So learn how to use
them. Place gears one over the other
so they can connect.

The pressure to gear component (the pipe with
a gear avove), will move the gear the same
ammounts of degrees that the pressure you
apply to it.

The gear valve, will distribute its inflow
(the leftmost connector), amongs its outputs
depending on how much degrees you rotate the 
gear on it.




















I cant always be funny.

"""
    levelWidth, levelHeight = 30,30
    
    test_cases = (
        dict(A=1, B=20),
        dict(A=10, B=5),
        dict(A=111, B=100),
        dict(A=100, B=1),
        dict(A=200, B=0),
        dict(A=333, B=400),
        dict(A=488, B=999),           
        )
                                
    ports = { 'A': ("left",5, PipeSlot, "data/labels/A.png"),
              'B': ("left",11, PipeSlot, "data/labels/B.png"),
              'AminusB': ("right",5, PipeSlot, "data/labels/AminusB.png"),
              "done": ("right", 11,PipeSlot, "data/labels/done.png"),
           }
    def machineValidator(self, machine):
        return (max(machine["A"].out_value-machine["B"].out_value,0)  == machine["AminusB"].in_value)
    
    def setPortA(self):
        return random.randint(0, 999)
    setPortB = setPortA
    
class LevelFive(Level):
    levelno = 5
    name = "Comparison"
    description = """Build a machine that produces at "equal"
a pressure of 1000 psi if the value of "A" is equal
to the value of "B".



"""
    levelWidth, levelHeight = 30,30
    
    test_cases = (
        dict(A=1, B=20),
        dict(A=10, B=5),
        dict(A=111, B=111),
        dict(A=100, B=1),
        dict(A=0, B=0),
        dict(A=333, B=400),
        dict(A=999, B=999),           
        )
                                
    ports = { 'A': ("left",5, PipeSlot, "data/labels/A.png"),
              'B': ("left",11, PipeSlot, "data/labels/B.png"),
              'equal': ("right",5, PipeSlot, "data/labels/equal.png"),
              "done": ("right", 11,PipeSlot, "data/labels/done.png"),
           }
    def machineValidator(self, machine):
        return ( 
            (machine["A"].out_value == machine["B"].out_value and machine["equal"].in_value == 1000)
            or
            ( machine["equal"].in_value != 1000 )
            )
    
    def setPortA(self):
        return random.randint(0, 999)
    setPortB = setPortA
    
class LevelSix(Level):
    levelno = 6
    name = "Multiplication"
    description = """
Build a machine that produces at "AtimesB" the 
value of "A" times the value of "B"

Remember to put 1000 pressure on "DONE" at the 
same step that "AtimesB" has the required value

For this machine you will need some more space,
so scroll to the sides to use the bigger
workspace

You will probably need to use a tank. A tank accumulates
its input and puts on the gear the value of
the pressure it has accumulated.

A big tank can hold 1000psi, a small tank can
hold 500psi. If you put too much pressure, the
tanks will blow up.

"""

    levelWidth, levelHeight = 60,60
    
    test_cases = (
        dict(A=1, B=1),
        dict(A=10, B=1),
        dict(A=1, B=10),
        dict(A=10, B=10),
        dict(A=200, B=0),
        dict(A=0, B=0),
        dict(A=33, B=22),           
        )
                                
    ports = { 'A': ("left",9, PipeSlot, "data/labels/A.png"),
              'B': ("left",15, PipeSlot, "data/labels/B.png"),
              "AtimesB": ("right",9, PipeSlot, "data/labels/AtimesB.png"),
              "done": ("right", 15,PipeSlot, "data/labels/done.png"),
           }
    def machineValidator(self, machine):
        return (machine["A"].out_value*machine["B"].out_value ==machine["AtimesB"].in_value)
    
    def setPortA(self):
        return random.randint(0, 30)
    setPortB = setPortA

if __name__ == "__main__":
    result = set()
    for c in locals().copy().values():
        if type(c) == type(Level) and issubclass(c, Level) and c != Level:
            for l in c.ports.keys():
                result.add(l)
                
    print result